@@ -3,7 +3,7 @@ use strict;
 use warnings;
 
 package Plack::Middleware::Rewrite;
-$Plack::Middleware::Rewrite::VERSION = '1.010';
+$Plack::Middleware::Rewrite::VERSION = '1.100';
 # ABSTRACT: mod_rewrite for Plack
 
 use parent 'Plack::Middleware';
@@ -12,8 +12,6 @@ use Plack::Util::Accessor 'rules';
 use Plack::Request ();
 use Plack::Util ();
 
-my %is_redir = map {; $_ => 1 } qw( 301 302 303 307 );
-
 sub call {
 	my $self = shift;
 	my ( $env ) = @_;
@@ -23,45 +21,41 @@ sub call {
 	# call rules with $_ aliased to PATH_INFO
 	my ( $res ) = map { $self->rules->( $env ) } $env->{'PATH_INFO'};
 
-	# return value fixup
-	if ( 'CODE' eq ref $res ) {
-		$modify_cb = $res;
-		undef $res;
-	}
-	elsif ( 'ARRAY' eq ref $res and @$res ) {
-		push @$res, [] if @$res < 2;
-		push @$res, [] if @$res < 3;
-	}
-	elsif ( not ref $res and defined $res and $res =~ /\A[1-5][0-9][0-9]\z/ ) {
-		$res = [ $res, [], [] ];
-	}
-	else {
-		undef $res;
-	}
+	# upgrade scalar return value, but only if it looks like an HTTP status
+	$res = [ $res, [], [] ]
+		if not ref $res
+		and defined $res
+		and $res =~ /\A[1-5][0-9][0-9]\z/;
 
-	if ( not $res ) { # redirect was internal
+	if ( 'ARRAY' eq ref $res and @$res ) { # external redirect, or explicit response
+		push @$res, map { [] } @$res .. 2;
+
+		if ( $res->[0] =~ /\A3[0-9][0-9]\z/ ) {
+			my $dest = Plack::Util::header_get( $res->[1], 'Location' );
+			if ( not $dest ) {
+				$dest = Plack::Request->new( $env )->uri;
+				Plack::Util::header_set( $res->[1], Location => $dest );
+			}
+
+			if ( 304 ne $res->[0] and not (
+				Plack::Util::content_length( $res->[2] )
+				or Plack::Util::header_exists( $res->[1], 'Content-Length' )
+			) ) {
+				my $href = Plack::Util::encode_html( $dest );
+				Plack::Util::header_set( $res->[1], qw( Content-Type text/html ) );
+				$res->[2] = [ qq'<!DOCTYPE html><title>Moved</title>This resource has moved to <a href="$href">a new address</a>.' ];
+			}
+		}
+	}
+	else { # internal redirect
+		$modify_cb = $res if 'CODE' eq ref $res;
 		$res = $self->app->( $env );
 	}
-	elsif ( $res->[0] =~ /\A3[0-9][0-9]\z/ ) {
-		my $dest = Plack::Util::header_get( $res->[1], 'Location' );
-		if ( not $dest ) {
-			$dest = Plack::Request->new( $env )->uri;
-			Plack::Util::header_set( $res->[1], Location => $dest );
-		}
-
-		if ( 304 ne $res->[0] and not (
-			Plack::Util::content_length( $res->[2] )
-			or Plack::Util::header_exists( $res->[1], 'Content-Length' )
-		) ) {
-			my $href = Plack::Util::encode_html( $dest );
-			Plack::Util::header_set( $res->[1], qw( Content-Type text/html ) );
-			$res->[2] = [ qq'<!DOCTYPE html><title>Moved</title>This resource has moved to <a href="$href">a new address</a>.' ];
-		}
-	}
 
 	return $res if not $modify_cb;
 	Plack::Util::response_cb( $res, sub {
-		$modify_cb->( $env ) for Plack::Util::headers( $_[0][1] );
+		my ( $res ) = map { $modify_cb->( $env ) } Plack::Util::headers( $_[0][1] );
+		return $res if 'CODE' eq ref $res;
 		return;
 	} );
 }
@@ -80,7 +74,7 @@ Plack::Middleware::Rewrite - mod_rewrite for Plack
 
 =head1 VERSION
 
-version 1.010
+version 1.100
 
 =head1 SYNOPSIS
 
@@ -106,7 +100,7 @@ version 1.010
              if $_ eq '/easter-egg';
 
          return sub { $_->set( 'Content-Type', 'application/xhtml+xml' ) }
-             if $_[0]{'HTTP_ACCEPT'} =~ m{application/xhtml\+xml(?!\s*;\s*q=0)}
+             if ( $_[0]{'HTTP_ACCEPT'} || '' ) =~ m{application/xhtml\+xml(?!\s*;\s*q=0)}
      };
      $app;
  };
@@ -162,6 +156,32 @@ with C<$_> aliased to a L<C<Plack::Util::headers>|Plack::Util> object for the
 response, for convenient alteration of headers. The L<PSGI> environment is,
 again, passed as its first and only argument.
 
+Any return value from this function will be ignored E<mdash>
+
+E<hellip> except if this function returns a function in turn. In that case, the
+function returned will be taken to be a filter for the body of the response
+about to be generated, with the interface documented in
+L<Plack::Middleware/RESPONSE CALLBACK>:
+
+=over 4
+
+ return sub {
+     my $res = shift;
+     return sub {
+         my $chunk = shift;
+         return unless defined $chunk;
+         $chunk =~ s/Foo/Bar/g;
+         return $chunk;
+     }
+ };
+
+The callback takes one argument C<$chunk> and your callback is expected to
+return the updated chunk. If the given C<$chunk> is undef, it means the stream
+has reached the end, so your callback should also return undef, or return the
+final chunk and return undef when called next time.
+
+=back
+
 =item Any other kind of value
 
 Other values are currently treated the same as returning nothing. This may
